import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { AdMapper } from '@modules/ad/ad.mapper';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { RpcValidationPipe } from '@mobicoop/ddd-library';
import { GRPC_SERVICE_NAME } from '@src/app.constants';
import { AdsResponseDto } from '../dtos/ads.response.dto';
import { FindAdsByUserIdQuery } from '@modules/ad/core/application/queries/find-ads-by-user-id/find-ads-by-user-id.query';
import { FindAdsByUserIdRequestDto } from './dtos/find-ads-by-user-id.request.dto';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class FindAdsByUserIdGrpcController {
  constructor(
    protected readonly mapper: AdMapper,
    private readonly queryBus: QueryBus,
  ) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'FindAllByUserId')
  async findAllByUserId(
    data: FindAdsByUserIdRequestDto,
  ): Promise<AdsResponseDto> {
    try {
      const ads: AdEntity[] = await this.queryBus.execute(
        new FindAdsByUserIdQuery(data.id),
      );
      return {
        ads: ads.map((ad: AdEntity) => this.mapper.toResponse(ad)),
      };
    } catch (e) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
